@@ -1,6 +1,6 @@
 /*
  *	endianttest.h - Platform endian testing
- *	Copyright (C) 2010, D Haley 
+ *	Copyright (C) 2011, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -29,11 +29,17 @@
 #ifdef __BYTE_ORDER
 //if both are not defined it is TRUE!
 #if __BYTE_ORDER == __BIG_ENDIAN
+#ifndef __BIG_ENDIAN__
 #define __BIG_ENDIAN__
+#endif
 #elif __BYTE_ORDER == __LITTLE_ENDIAN
+#ifndef __LITTLE_ENDIAN__
 #define __LITTLE_ENDIAN__
+#endif
 #elif __BYTE_ORDER == __PDP_ENDIAN
+#ifndef __ARM_ENDIAN__
 #define __ARM_ENDIAN__
+#endif
 #else
 #error "Endian determination failed"
 #endif
