@@ -6,7 +6,7 @@
 #include "ionClip.h"
 #include "ionColour.h"
 #include "ionDownsample.h"
-#include "posLoad.h"
+#include "dataLoad.h"
 #include "rangeFile.h"
 #include "clusterAnalysis.h"
 #include "spatialAnalysis.h"
@@ -14,6 +14,7 @@
 #include "transform.h"
 #include "voxelise.h"
 #include "ionInfo.h"
+#include "annotation.h"
 
 //!Create a "true default" filter from its true name string
 Filter *makeFilter(const string  &s) ;
