@@ -115,7 +115,7 @@ unsigned int doHull(unsigned int bufferSize, double *buffer,
 }
 
 bool getRectilinearBounds(const std::vector<const FilterStreamData *> &dataIn, BoundCube &bound,
-					unsigned int *progress, unsigned int totalSize,bool (*callback)(void))
+					unsigned int *progress, unsigned int totalSize,bool (*callback)(bool))
 {
 	bound.setInvalid();
 
@@ -154,7 +154,7 @@ bool getRectilinearBounds(const std::vector<const FilterStreamData *> &dataIn, B
 			}
 		
 			*progress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
-			if(!(*callback)())
+			if(!(*callback)(false))
 				return false;
 		}
 	}
@@ -245,7 +245,7 @@ void IonInfoFilter::initFilter(const std::vector<const FilterStreamData *> &data
 //TODO: Refactor -- this is a clone of countPoints from voxelise.cpp
 //This is not a member of voxels.h, as the voxels do not have any concept of the IonHit
 int countPoints(Voxels<size_t> &v, const std::vector<IonHit> &points, 
-				bool noWrap,bool (*callback)(void))
+				bool noWrap,bool (*callback)(bool))
 {
 
 	size_t x,y,z;
@@ -257,7 +257,7 @@ int countPoints(Voxels<size_t> &v, const std::vector<IonHit> &points,
 	{
 		if(!downSample--)
 		{
-			if(!(*callback)())
+			if(!(*callback)(false))
 				return 1;
 			downSample=MAX_CALLBACK;
 		}
@@ -266,18 +266,12 @@ int countPoints(Voxels<size_t> &v, const std::vector<IonHit> &points,
 		//Ensure it lies within the dataset
 		if (x < binCount[0] && y < binCount[1] && z< binCount[2])
 		{
-			{
-				float value;
-				value=v.getData(x,y,z)+1.0f;
+			size_t value;
+			value=v.getData(x,y,z);
 
 				//Prevent wrap-around errors
-				if (noWrap) {
-					if (value > v.getData(x,y,z))
-						v.setData(x,y,z,value);
-				} else {
-					v.setData(x,y,z,value);
-				}
-			}
+			if(!noWrap || value != std::numeric_limits<size_t>::max())
+				v.setData(x,y,z,value+1);
 		}
 	}
 	return 0;
@@ -304,7 +298,7 @@ Filter *IonInfoFilter::cloneUncached() const
 }
 
 unsigned int IonInfoFilter::refresh(const std::vector<const FilterStreamData *> &dataIn,
-	std::vector<const FilterStreamData *> &getOut, ProgressData &progress, bool (*callback)(void))
+	std::vector<const FilterStreamData *> &getOut, ProgressData &progress, bool (*callback)(bool))
 {
 
 	//Count the number of ions input
@@ -522,21 +516,13 @@ void IonInfoFilter::getProperties(FilterProperties &propertyList) const
 		keys.push_back(KEY_NORMALISE);
 		type.push_back(PROPERTY_TYPE_BOOL);
 
-		propertyList.data.push_back(s);
-		propertyList.keys.push_back(keys);
-		propertyList.types.push_back(type);
-		s.clear(); keys.clear(); type.clear();
-	} 
-	
-	if(wantVolume) 	
-	{
-		propertyList.data.push_back(s);
-		propertyList.keys.push_back(keys);
-		propertyList.types.push_back(type);
-
-		s.clear();keys.clear();type.clear();
 	}
 
+	propertyList.data.push_back(s);
+	propertyList.keys.push_back(keys);
+	propertyList.types.push_back(type);
+	s.clear(); keys.clear(); type.clear();
+
 	stream_cast(str,wantVolume);
 	keys.push_back(KEY_VOLUME);
 	s.push_back(make_pair(TRANS("Volume"), str));
@@ -661,7 +647,7 @@ std::string  IonInfoFilter::getErrString(unsigned int code) const
 }
 
 unsigned int IonInfoFilter::convexHullEstimateVol(const vector<const FilterStreamData*> &data, 
-								float &volume,bool (*callback)())const
+								float &volume,bool (*callback)(bool))const
 {
 	//OK, so heres the go. partition the input data
 	//into GRAB_SIZE lots before processing hull.
@@ -752,7 +738,7 @@ unsigned int IonInfoFilter::convexHullEstimateVol(const vector<const FilterStrea
 			}
 
 
-			if(!(*callback)())
+			if(!(*callback)(false))
 			{
 				free(buffer);
 				return ERR_USER_ABORT;
@@ -764,6 +750,14 @@ unsigned int IonInfoFilter::convexHullEstimateVol(const vector<const FilterStrea
 
 	}
 
+    if(doneHull)
+    {
+        //Free the last convex hull mem	
+        qh_freeqhull(!qh_ALL);
+        
+        int curlong, totlong;
+        qh_memfreeshort(&curlong, &totlong);
+    }
 
 	//Need at least 3 objects to construct a sufficiently large buffer
 	if(bufferSize + curHull.size() > 3)
@@ -870,7 +864,7 @@ bool IonInfoFilter::writeState(std::ofstream &f,unsigned int format, unsigned in
 		case STATE_FORMAT_XML:
 		{	
 			f << tabs(depth) <<  "<" << trueName() << ">" << endl;
-			f << tabs(depth+1) << "<userstring value=\""<<userString << "\"/>"  << endl;
+			f << tabs(depth+1) << "<userstring value=\""<< escapeXML(userString) << "\"/>"  << endl;
 
 			f << tabs(depth+1) << "<wantioncounts value=\""<<wantIonCounts<< "\"/>"  << endl;
 			f << tabs(depth+1) << "<wantnormalise value=\""<<wantNormalise<< "\"/>"  << endl;
@@ -998,7 +992,7 @@ void makeSphereOutline(float radius, float angularStep,
 {
 	d->clear();
 	ASSERT(angularStep > 0.0f);
-	unsigned int numAngles= 180.0/angularStep;
+	unsigned int numAngles=(unsigned int)( 180.0f/angularStep);
 
 	for( unsigned int  ui=0; ui<numAngles; ui++)
 	{
@@ -1145,7 +1139,8 @@ bool volumeSphereTest()
 	vector<string> dummy;
 	f->getConsoleStrings(dummy);
 
-	f->refresh(streamIn,streamOut,p,dummyCallback);
+	TEST(!f->refresh(streamIn,streamOut,p,dummyCallback),"refresh error code");
+
 	volMeasure=f->getLastVolume();
 
 	//Convex volume of sphere
