@@ -22,9 +22,11 @@
 #include <wx/glcanvas.h>
 #include <wx/progdlg.h>
 #include <wx/defs.h>
+#include "wxcommon.h"
 
 #include "glPane.h"
 
+
 // include OpenGL
 #ifdef __WXMAC__
 #include "OpenGL/glu.h"
@@ -94,6 +96,7 @@ wxGLCanvas(parent, wxID_ANY,  wxDefaultPosition, wxDefaultSize, 0, wxT("GLCanvas
 	haveCameraUpdates=false;
 	applyingDevice=false;
 	paneInitialised=false;
+	disableSceneInteraction=false;
 
 	keyDoubleTapTimer=new wxTimer(this,ID_KEYPRESS_TIMER);
 	lastKeyDoubleTap=(unsigned int)-1;
@@ -116,13 +119,18 @@ bool BasicGLPane::displaySupported() const
 	return IsDisplaySupported(attribList);
 #else
 	ASSERT(false);
-	//Lets hope so. If its not, then its just going to fai anyway. 
+	//Lets hope so. If its not, then its just going to fail anyway. 
 	//If it is, then returning false would simply create a roadblock.
 	//Either way, you shouldn't get here.
 	return true; 
 #endif
 }
 
+void BasicGLPane::setSceneInteractionAllowed(bool enabled)
+{
+	disableSceneInteraction=!enabled;
+}
+
 unsigned int  BasicGLPane::selectionTest(wxPoint &p,bool &shouldRedraw)
 {
 
@@ -208,8 +216,15 @@ void BasicGLPane::mouseMoved(wxMouseEvent& event)
 	};
 
 
-	if(selectionMode)
+	if(selectionMode )
 	{
+		if(disableSceneInteraction)
+		{
+			event.Skip();
+			return;
+		}
+
+
 		wxPoint p=event.GetPosition();
 		
 		unsigned int mouseFlags=0;
@@ -239,6 +254,7 @@ void BasicGLPane::mouseMoved(wxMouseEvent& event)
 		int w, h;
 		GetClientSize(&w, &h);
 
+
 		currentScene.applyDevice((float)draggingStart.x/(float)w,
 					(float)draggingStart.y/(float)h,
 					 p.x/(float)w,p.y/(float)h,
@@ -297,8 +313,8 @@ void BasicGLPane::mouseMoved(wxMouseEvent& event)
 	lrMove=CAMERA_MOVE_RATE*camMultRate*(draggingCurrent.x - draggingStart.x);
 	udMove=CAMERA_MOVE_RATE*camMultRate*(draggingCurrent.y - draggingStart.y);
 
-	lrMove*=2*M_PI/180.0;
-	udMove*=2*M_PI/180.0;
+	lrMove*=2.0f*M_PI/180.0;
+	udMove*=2.0f*M_PI/180.0;
 	unsigned int camMode=0;
 	//Decide camera movement mode
 	bool translateMode;
@@ -369,9 +385,9 @@ void BasicGLPane::mouseDown(wxMouseEvent& event)
 
 	//Do not re-trigger if dragging or doing a scene update.
 	//This can cause a selection test to occur whilst
-	//a temp cam is activated in the scene,
+	//a temp cam is activated in the scene, or a binding refresh is underway,
 	//which is currently considered bad
-	if(!dragging && !applyingDevice && !selectionMode)
+	if(!dragging && !applyingDevice && !selectionMode && !disableSceneInteraction)
 	{
 		//Check to see if the user has clicked an object in the scene
 		bool redraw;
@@ -517,7 +533,6 @@ void BasicGLPane::mouseLeftWindow(wxMouseEvent& event)
 	{
 		if(currentScene.haveTempCam())
 		{
-			wxPoint draggingCurrent = event.GetPosition();
 			currentScene.commitTempCam();
 			dragging=false;
 		}
@@ -598,11 +613,7 @@ void BasicGLPane::keyPressed(wxKeyEvent& event)
 
 				
 				currentScene.ensureVisible(visibleDir);
-#ifdef wxMAC
-				parentStatusBar->SetStatusText(wxTRANS("Use shift/⌘-space or double tap to alter reset axis"));
-#else
 				parentStatusBar->SetStatusText(wxTRANS("Use shift/ctrl-space or double tap to alter reset axis"));
-#endif
 				parentStatusBar->SetBackgroundColour(*wxCYAN)
 					;
 				parentStatusTimer->Start(statusDelay,wxTIMER_ONE_SHOT);
@@ -829,13 +840,15 @@ bool BasicGLPane::saveImage(unsigned int width, unsigned int height,
 	int panelWidth,panelHeight;
 	GetClientSize(&panelWidth,&panelHeight);
 
-	
+	float oldAspect = currentScene.getAspect();
+	currentScene.setAspect((float)panelHeight/(float)panelWidth);
+
 	//Check
 	if((unsigned int)width > panelWidth || (unsigned int)height> panelHeight)
 	{
 		unsigned int numTilesX,numTilesY;
 		numTilesX = width/panelWidth;
-		numTilesY = width/panelWidth;
+		numTilesY = height/panelHeight;
 		if(panelWidth % width)
 			numTilesX++;
 		
@@ -981,8 +994,12 @@ bool BasicGLPane::saveImage(unsigned int width, unsigned int height,
 
 	bool isOK=image->SaveFile(wxCStr(filename),wxBITMAP_TYPE_PNG);
 
+	currentScene.setAspect(oldAspect);
 	delete image;
 
+	wxPaintEvent event;
+	wxPostEvent(this,event);
+
 	return isOK;
 }
 
@@ -1045,6 +1062,9 @@ bool BasicGLPane::saveImageSequence(unsigned int resX, unsigned int resY, unsign
 	//Discard the current temp. cam to return the scene back to normal
 	currentScene.discardTempCam();
 	wxD->Destroy();
+	
+	wxPaintEvent event;
+	wxPostEvent(this,event);
 	return true;
 		
 }
