@@ -18,8 +18,20 @@
 
 #include "rdf.h"
 #include "mathfuncs.h"
-#include "voxels.h"
 
+#include <utility>
+
+using std::vector;
+#ifdef DEBUG
+#include <iostream>
+#include <cstdlib>
+using std::cerr;
+using std::endl;
+#endif
+
+#ifdef _OPENMP
+#include <omp.h> 
+#endif
 //QHull library
 //Build fix for qhull ; wx defines powerpc without
 //assigning a value, causing build fail on powerpc
@@ -66,11 +78,9 @@ unsigned int vectorPointDir(const Point3D &pA, const Point3D &pB,
 	if( dot1 < 0.0f && dot2 <0.0f )
 		return POINTDIR_APART; 
 
-	if(dot1 > 0.0f && dot2 > 0.0f )
-		return POINTDIR_TOGETHER;
-
-	ASSERT(false)
-	return 0; 
+	ASSERT(dot1 > 0.0f && dot2 > 0.0f );
+	
+	return POINTDIR_TOGETHER;
 }
 
 //! Returns the shortest distance between a line segment and a given point
@@ -333,19 +343,14 @@ unsigned int GetReducedHullPts(const vector<Point3D> &points, float reductionDim
 	//shrink the convex hull such that it lies at
 	//least reductionDim from the original surface of
 	//the convex hull
-	//The scale factor that we want is 
-	//given by s = 1 -(reductionDim/maxDist)
 	float scaleFactor;
 	scaleFactor = 1  - reductionDim/ minDist;
 
 	if(scaleFactor < 0.0f)
-	{
-		cerr << "Scale factor negative. Aborting." << endl;
 		return RDF_ERR_NEGATIVE_SCALE_FACT;
-	}
 
 	//update mindist such that it is still valid after scaling
-	minDist=minDist*scaleFactor;
+	minDist=minDist*scaleFactor; 
 	
 #ifdef DEBUG
 	cerr << "Scale Factor : " << scaleFactor << endl;
@@ -434,7 +439,7 @@ reduced_loop_next:
 unsigned int generateNNHist( const vector<Point3D> &pointList, 
 			const K3DTree &tree,unsigned int nnMax, unsigned int numBins,
 		       	vector<vector<size_t> > &histogram, float *binWidth , unsigned int *progressPtr,
-			bool (*callback)(void))
+			bool (*callback)(bool))
 {
 	if(pointList.size() <=nnMax)
 		return RDF_ERR_INSUFFICIENT_INPUT_POINTS;
@@ -454,17 +459,13 @@ unsigned int generateNNHist( const vector<Point3D> &pointList,
 	
 
 
-	size_t numAnalysed=0;
-#ifdef _OPENMP
-	bool spin=false;
-	int callbackReduce[omp_get_num_threads()];
-	for(unsigned int ui=0;ui<omp_get_num_threads();ui++)
-		callbackReduce[ui]=CALLBACK_REDUCE;
-#else
 	int callbackReduce=CALLBACK_REDUCE;
+#ifdef _OPENMP
+	size_t numAnalysed=0;
+	bool spin=false;
 #endif
 	//do NN search
-#pragma omp parallel for shared(spin,callbackReduce)
+#pragma omp parallel for shared(spin,numAnalysed) firstprivate(callbackReduce)
 	for(unsigned int ui=0; ui<pointList.size(); ui++)
 	{
 #ifdef _OPENMP
@@ -487,30 +488,26 @@ unsigned int generateNNHist( const vector<Point3D> &pointList,
 			
 
 		//Callbacks to perform UI updates as needed
-#ifdef _OPENMP 
-		if(!(callbackReduce[omp_get_thread_num()]--))
-		{
-		#pragma omp critical
-		{
-			*progressPtr= (unsigned int)((float)(numAnalysed)/((float)pointList.size())*100.0f);
-			if(!(*callback)())
-				spin=true;
-			numAnalysed+=CALLBACK_REDUCE;
-		}			
-			callbackReduce[omp_get_thread_num()]=CALLBACK_REDUCE;
-		}	
-#else
 		if(!(callbackReduce--))
 		{
-			*progressPtr= (unsigned int)((float)(numAnalysed)/((float)pointList.size())*100.0f);
-			if(!(*callback)())
+#ifdef _OPENMP 
+			#pragma omp critical
+			{
+				numAnalysed+=CALLBACK_REDUCE;
+				*progressPtr= (unsigned int)((float)(numAnalysed)/((float)pointList.size())*100.0f);
+				if(!(*callback)(false))
+					spin=true;
+			}			
+#else
+			*progressPtr= (unsigned int)((float)(ui)/((float)pointList.size())*100.0f);
+			if(!(*callback)(false))
 			{
 				delete[] maxSqrDist;
 				return RDF_ABORT_FAIL;
 			}
+#endif
 			callbackReduce=CALLBACK_REDUCE;
 		}
-#endif
 
 	}
 #ifdef _OPENMP
@@ -535,12 +532,12 @@ unsigned int generateNNHist( const vector<Point3D> &pointList,
 	}	
 
 	maxOfMaxDists=sqrtf(maxOfMaxDists);
-	maxDist[nnMax] = maxOfMaxDists;	
+	maxDist[nnMax-1] = maxOfMaxDists;	
 
 	//Cacluate the bin widths required to accommodate this
-	//distribution, allowing an extra 2% for the tail bin
+	//distribution
 	for(unsigned int ui=0; ui<nnMax; ui++)
-		binWidth[ui]= 1.02f*maxDist[ui]/(float)numBins;
+		binWidth[ui]= maxDist[ui]/(float)numBins;
 	delete[] maxDist;
 	
 	//Generate histogram for what we have already
@@ -555,12 +552,10 @@ unsigned int generateNNHist( const vector<Point3D> &pointList,
 #ifdef _OPENMP
 	spin=false;
 	numAnalysed=0;
-	for(unsigned int ui=0;ui<omp_get_num_threads();ui++)
-		callbackReduce[ui]=CALLBACK_REDUCE;
-#else
-	callbackReduce=CALLBACK_REDUCE;
 #endif
-#pragma omp parallel for
+	
+	callbackReduce=CALLBACK_REDUCE;
+#pragma omp parallel for firstprivate(callbackReduce)
 	for(unsigned int ui=0; ui<pointList.size(); ui++)
 	{
 		vector<const Point3D *> nnPoints;
@@ -591,22 +586,22 @@ unsigned int generateNNHist( const vector<Point3D> &pointList,
 
 		//Callbacks to perform UI updates as needed
 #ifdef _OPENMP 
-		if(!(callbackReduce[omp_get_thread_num()]--))
+		if(!(callbackReduce--))
 		{
 		#pragma omp critical
 		{
 			*progressPtr= (unsigned int)((float)(numAnalysed)/((float)pointList.size())*100.0f);
-			if(!(*callback)())
+			if(!(*callback)(false))
 				spin=true;
 			numAnalysed+=CALLBACK_REDUCE;
 		}			
-			callbackReduce[omp_get_thread_num()]=CALLBACK_REDUCE;
+			callbackReduce=CALLBACK_REDUCE;
 		}	
 #else
 		if(!(callbackReduce--))
 		{
-			*progressPtr= (unsigned int)((float)(numAnalysed)/((float)pointList.size())*100.0f);
-			if(!(*callback)())
+			*progressPtr= (unsigned int)((float)(ui)/((float)pointList.size())*100.0f);
+			if(!(*callback)(false))
 			{
 				delete[] maxSqrDist;
 				return RDF_ABORT_FAIL;
@@ -626,11 +621,18 @@ unsigned int generateNNHist( const vector<Point3D> &pointList,
 unsigned int generateDistHist(const vector<Point3D> &pointList, const K3DTree &tree,
 			unsigned int *histogram, float distMax,
 			unsigned int numBins, unsigned int &warnBiasCount,
-			unsigned int *progressPtr,bool (*callback)(void))
+			unsigned int *progressPtr,bool (*callback)(bool))
 		
 {
 
 
+#ifdef DEBUG
+	for(unsigned int ui=0;ui<numBins;ui++)
+	{
+		ASSERT(!histogram[ui]);
+	}
+#endif
+
 	if(pointList.empty())
 		return 0;
 
@@ -642,20 +644,24 @@ unsigned int generateDistHist(const vector<Point3D> &pointList, const K3DTree &t
 	//but i dont have  a tree.numvertcies
 	float maxSqrDist = distMax*distMax;
 
-	vector<const Point3D *> pts;
 	warnBiasCount=0;
 	
 	//Main r-max searching routine
+	unsigned int callbackReduce=CALLBACK_REDUCE;
 #ifdef _OPENMP
 	bool spin=false;
 	size_t numAnalysed=0;
-	int callbackReduce[omp_get_num_threads()];
-	for(unsigned int ui=0;ui<omp_get_num_threads();ui++)
-		callbackReduce[ui]=CALLBACK_REDUCE;
-#else
-	int callbackReduce=CALLBACK_REDUCE;
+	size_t numAnalysedThread=0;
+
+	unsigned int threadHist[numBins][omp_get_max_threads()];
+	for(size_t ui=0; ui<omp_get_max_threads(); ui++)
+	{
+		for (size_t uj = 0; uj < numBins; uj++)
+			threadHist[uj][ui] = 0;
+	}
 #endif
-#pragma omp parallel for shared(spin,callbackReduce,histogram)
+
+#pragma omp parallel for shared(spin,histogram,numAnalysed,threadHist) firstprivate(callbackReduce,numAnalysedThread) default(shared)
 	for(unsigned int ui=0; ui<pointList.size(); ui++)
 	{
 #ifdef _OPENMP
@@ -663,7 +669,7 @@ unsigned int generateDistHist(const vector<Point3D> &pointList, const K3DTree &t
 			continue;
 #endif
 
-		pair<unsigned int,unsigned int> thisPair;	
+		std::pair<unsigned int,unsigned int> thisPair;	
 		float sqrDist,deadDistSqr;
 		Point3D sourcePoint;
 		const Point3D *nearPt=0;
@@ -695,13 +701,11 @@ unsigned int generateDistHist(const vector<Point3D> &pointList, const K3DTree &t
 				if(sqrDist < maxSqrDist)
 				{
 					//Add the point to the histogram
-					unsigned int *bin;
-					unsigned int offset;
-					offset=((size_t) ((sqrtf(sqrDist/maxSqrDist)*(float)numBins)));
-					bin= histogram+offset;
-
-#pragma omp atomic
-					(*bin)++;
+#ifdef _OPENMP
+					threadHist[(size_t) ((sqrtf(sqrDist/maxSqrDist)*(float)numBins))] [omp_get_thread_num()]++;
+#else
+					histogram[(size_t)((sqrtf(sqrDist/maxSqrDist)*(float)numBins))]++;
+#endif
 				}
 
 				//increase the dead distance to the last distance
@@ -715,34 +719,41 @@ unsigned int generateDistHist(const vector<Point3D> &pointList, const K3DTree &t
 			}
 
 
-#ifdef _OPENMP 
-			if(!(callbackReduce[omp_get_thread_num()]--))
-			{
-			#pragma omp critical
-			{
-				*progressPtr= (unsigned int)((float)(numAnalysed)/((float)pointList.size())*100.0f);
-				if(!(*callback)())
-					spin=true;
-				numAnalysed+=CALLBACK_REDUCE;
-			}			
-				callbackReduce[omp_get_thread_num()]=CALLBACK_REDUCE;
-			}	
-#else
 			if(!(callbackReduce--))
 			{
+#ifdef _OPENMP 
+				#pragma omp critical
+				{
+					numAnalysed+=numAnalysedThread;
+					*progressPtr= (unsigned int)((float)(numAnalysed)/((float)pointList.size())*100.0f);
+					if(!(*callback)(false))
+						spin=true;
+				}
+				if(spin)
+					break;
+				numAnalysedThread=0;
+#else
 				*progressPtr= (unsigned int)((float)(ui)/((float)pointList.size())*100.0f);
-				if(!(*callback)())
+				if(!(*callback)(false))
 					return RDF_ABORT_FAIL;
+#endif
 				callbackReduce=CALLBACK_REDUCE;
 			}
-#endif
 		}
-
+#ifdef _OPENMP
+		numAnalysedThread++;
+#endif
 	}
 
 #ifdef _OPENMP
 	if(spin)
 		return RDF_ABORT_FAIL;
+    
+	for (size_t i = 0; i < numBins; i++)
+	{
+		for (size_t j = 0; j < omp_get_max_threads(); j++)
+			histogram[i] += threadHist[i][j];
+	}
 #endif
 
 	//Calculations complete!
