@@ -545,12 +545,13 @@ void Scene::clearObjs()
 	for(unsigned int ui=0; ui<objects.size(); ui++)
 		delete objects[ui];
 	objects.clear();
+	lastHovered=-1;
 }
 
 void Scene::clearBindings()
 {
-	for(unsigned int ui=0; ui<selectionDevices.size(); ui++)
-		delete selectionDevices[ui];
+	//Note that the filter is responsible for cleaning the
+	// bindings. So, we just forget about it, without delete-ing
 	selectionDevices.clear();
 }
 
