@@ -1,6 +1,6 @@
 /*
  * 	select,h - Opengl interaction header. 
- *	Copyright (C) 2010, D Haley 
+ *	Copyright (C) 2011, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -198,18 +198,39 @@ template<class T>
 bool SelectionDevice<T>::getBinding(const DrawableObj *d,unsigned int mouseFlags, 
 					unsigned int keyFlags,SelectionBinding* &b)
 {
+
+	unsigned int keyMask=0;
+
+
+	bool found=false;
+
 	for(unsigned int ui=0;ui<bindingVec.size();ui++)
 	{
 		if(bindingVec[ui].matchesDrawable(d,mouseFlags,keyFlags))
 		{
-			b=&(bindingVec[ui]);
-			return true;
+			if(!found)
+			{
+				//we found one.
+				found=true;
+				b=&(bindingVec[ui]);
+				keyMask=b->getKeyFlags();
+				continue;
+			}
+
+			//OK, we already have one, but we can be "trumped"
+			//by a more complex keymask.
+			if( (keyMask & bindingVec[ui].getKeyFlags() )== keyMask)
+			{
+				b=&(bindingVec[ui]);
+				keyMask=b->getKeyFlags();
+			}
 		}
 	}
 
+
 	//This selection device does not match
 	//the targeted object.
-	return false;
+	return found;
 }
 
 template<class T>
